@@ -55,9 +55,6 @@ struct opt_data {
 	struct acvp_rename_ctx rename_ctx;
 	char *specific_modversion;
 
-	char *seed_base64;
-	uint32_t seed_base64_len;
-
 	char *configfile;
 	struct json_object *config;
 	const char *tlskey;
@@ -66,6 +63,10 @@ struct opt_data {
 	const char *tlscertkeychainref;
 	const char *tlscabundle;
 	const char *tlscakeychainref;
+	const char *seedfile;
+	const char *cert_details_niap_req_file;
+	const char *acvp_server_db_search;
+	enum acvp_server_db_search_type search_type;
 	char *basedir;
 	char *secure_basedir;
 	char *definition_basedir;
@@ -183,11 +184,9 @@ static int read_complete(int fd, char *buf, uint32_t buflen)
 
 static int load_config(struct opt_data *opts)
 {
-	struct stat statbuf;
 	struct flock lock;
 	int ret;
 	int fd;
-	const char *seedfile;
 
 	fd = open(global_opts->configfile, O_RDONLY);
 	if (fd < 0) {
@@ -256,46 +255,64 @@ static int load_config(struct opt_data *opts)
 	if (ret)
 		opts->tlscakeychainref = NULL;
 
-	CKINT(json_get_string(opts->config, OPT_STR_TOTPSEEDFILE, &seedfile,
-			      false));
+	CKINT(json_get_string(opts->config, OPT_STR_TOTPSEEDFILE,
+			      &opts->seedfile, false));
 
-	if (stat(seedfile, &statbuf)) {
+out:
+	if (fd >= 0)
+		close(fd);
+	return ret;
+}
+
+static int load_totp_seed(struct opt_data *opts, char **seed_base64,
+			  uint32_t *seed_base64_len)
+{
+	struct stat statbuf;
+	char *seed;
+	uint32_t len;
+	int ret;
+	int fd = -1;
+
+	if (stat(opts->seedfile, &statbuf)) {
 		ret = -errno;
 		logger(LOGGER_ERR, LOGGER_C_ANY,
-		       "Error accessing seed file %s (%d)\n", seedfile,
+		       "Error accessing seed file %s (%d)\n", opts->seedfile,
 		       ret);
 		goto out;
 	}
 
 	if (!statbuf.st_size) {
-		logger(LOGGER_ERR, LOGGER_C_ANY, "Seed file %s empty (%d)\n",
-		       seedfile, ret);
+		logger(LOGGER_ERR, LOGGER_C_ANY, "Seed file %s empty\n",
+		       opts->seedfile);
 		ret = -EINVAL;
 		goto out;
 	}
 
-	fd = open(seedfile, O_RDONLY | O_CLOEXEC);
+	fd = open(opts->seedfile, O_RDONLY | O_CLOEXEC);
 	if (fd < 0) {
 		ret = -errno;
 		logger(LOGGER_ERR, LOGGER_C_ANY,
-		       "Cannot open seed file %s (%d)\n", seedfile, ret);
+		       "Cannot open seed file %s (%d)\n", opts->seedfile, ret);
 		goto out;
 	}
 
-	opts->seed_base64_len = (uint32_t)statbuf.st_size;
-	opts->seed_base64 = malloc((size_t)statbuf.st_size);
-	if (!opts->seed_base64) {
+	len = (uint32_t)statbuf.st_size;
+	seed = malloc((size_t)statbuf.st_size);
+	if (!seed) {
 		ret = -ENOMEM;
 		goto out;
 	}
 
-	CKINT(read_complete(fd, opts->seed_base64, opts->seed_base64_len));
+	CKINT(read_complete(fd, seed, len));
 
-	while (isspace(opts->seed_base64[opts->seed_base64_len - 1]))
-		opts->seed_base64_len--;
+	while (isspace(seed[len - 1]))
+		len--;
 
 	logger(LOGGER_DEBUG, LOGGER_C_ANY,
-	       "TOTP seed file %s read into memory\n", seedfile);
+	       "TOTP seed file %s read into memory\n", opts->seedfile);
+
+	*seed_base64 = seed;
+	*seed_base64_len = len;
 
 out:
 	if (fd >= 0)
@@ -318,9 +335,8 @@ static void usage(void)
 	fprintf(stderr, "Upload test responses and (continue to) fetch verdict:\n");
 	fprintf(stderr, "  acvp-proxy [-mrnepf MODULE_SEARCH_CRITERIA] [--testid|--vsid ID]\n\n");
 
-	//TODO issue #518: remove caveat once issue is cleared
-	fprintf(stderr, "Download samples after vectors are obtained (CURRENTLY NOT SUPPORTED BY ACVP SERVER):\n");
-	fprintf(stderr, "  acvp-proxy [-mrnepf MODULE_SEARCH_CRITERIA] --request [--testid|--vsid ID] --sample\n\n");
+	fprintf(stderr, "Download samples after vectors are obtained:\n");
+	fprintf(stderr, "  acvp-proxy [-mrnepf MODULE_SEARCH_CRITERIA] [--testid|--vsid ID] --sample\n\n");
 	fprintf(stderr, "Usage:\n");
 	fprintf(stderr, "\tModule search criteria limiting the scope of processed modules:\n");
 	fprintf(stderr, "\t-m --module <NAME>\t\tDefinition search criteria: Name of\n");
@@ -408,9 +424,19 @@ static void usage(void)
 	fprintf(stderr, "\t   --list-certificates\t\tList all certificates\n");
 	fprintf(stderr, "\t   --list-cert-details\t\tList all certificate details for\n");
 	fprintf(stderr, "\t\t\t\t\tTE.01.12.01\n");
+	fprintf(stderr, "\t   --list-cert-niap <FILE>\tList all certificate details for\n");
+	fprintf(stderr, "\t\t\t\t\ta NIAP CC eval\n");
 	fprintf(stderr, "\t   --list-cipher-options\tList all cipher options\n");
 	fprintf(stderr, "\t   --list-cipher-options-deps\tList all cipher options with\n");
-	fprintf(stderr, "\t\t\t\t\tcipher dependencies\n\n");
+	fprintf(stderr, "\t\t\t\t\tcipher dependencies\n");
+	fprintf(stderr, "\t   --list-server-db <TYPE>\tList entries in ACVP server database\n");
+	fprintf(stderr, "\t\t\t\t\tTYPE: [oe|vendor|module|person]\n");
+	fprintf(stderr, "\t   --search-server-db <SEARCH>\tSearch ACVP server database\n");
+	fprintf(stderr, "\t\t\t\t\tSEARCH: <TYPE>:<QUERY>\n");
+	fprintf(stderr, "\t\t\t\t\tTYPE:\t[oe|vendor|module|person|\n");
+	fprintf(stderr, "\t\t\t\t\t\taddress|dependency]\n");
+	fprintf(stderr, "\t\t\t\t\tQUERY: string as defined in ACVP spec\n");
+	fprintf(stderr, "\t\t\t\t\tsection 11.6 \n\n");
 
 	fprintf(stderr, "\tGathering cipher definitions from ACVP server:\n");
 	fprintf(stderr, "\t   --cipher-list\t\tList all ciphers supported by ACVP\n");
@@ -422,6 +448,7 @@ static void usage(void)
 	fprintf(stderr, "\tAuxiliary options:\n");
 	fprintf(stderr, "\t   --proxy-extension <SO-FILE>\tShared library of ACVP Proxy extension\n");
 	fprintf(stderr, "\t\t\t\t\tZero or more extensions can be provided.\n");
+	fprintf(stderr, "\t   --proxy-extension-dir <DIR>\tDirectory with ACVP Proxy extensions\n");
 	fprintf(stderr, "\t   --rename-version <NEW>\tRename version of definition\n");
 	fprintf(stderr, "\t\t\t\t\t(moduleVersion)\n");
 	fprintf(stderr, "\t   --rename-name <NEW>\t\tRename name of definition (moduleName)\n");
@@ -464,8 +491,6 @@ static void free_opts(struct opt_data *opts)
 		free(search->processor);
 	if (opts->specific_modversion)
 		free(opts->specific_modversion);
-	if (opts->seed_base64)
-		free(opts->seed_base64);
 	if (opts->configfile)
 		free(opts->configfile);
 	if (opts->basedir)
@@ -547,6 +572,56 @@ static int convert_update_delete_type(const char *string, unsigned int *option)
 	return 0;
 }
 
+static int convert_show_type(const char *string, unsigned int *option)
+{
+	if (!strncmp(string, "oe", 2)) {
+		*option |= ACVP_OPTS_SHOW_OE;
+	} else if (!strncmp(string, "vendor", 6)) {
+		*option |= ACVP_OPTS_SHOW_VENDOR;
+	} else if (!strncmp(string, "person", 6)) {
+		*option |= ACVP_OPTS_SHOW_PERSON;
+	} else if (!strncmp(string, "module", 6)) {
+		*option |= ACVP_OPTS_SHOW_MODULE;
+	} else {
+		logger(LOGGER_ERR, LOGGER_C_ANY,
+		"Unknown show type %s\n", string);
+		return -EINVAL;
+	}
+
+	return 0;
+}
+
+static int convert_search_type(const char *string,
+			       const char **searchstr,
+			       enum acvp_server_db_search_type *type)
+{
+	if (!strncmp(string, "oe:", 3)) {
+		*type = NIST_SERVER_DB_SEARCH_OE;
+		*searchstr = string + 3;
+	} else if (!strncmp(string, "vendor:", 7)) {
+		*type = NIST_SERVER_DB_SEARCH_VENDOR;
+		*searchstr = string + 7;
+	} else if (!strncmp(string, "person:", 7)) {
+		*type = NIST_SERVER_DB_SEARCH_PERSONS;
+		*searchstr = string + 7;
+	} else if (!strncmp(string, "module:", 7)) {
+		*type = NIST_SERVER_DB_SEARCH_MODULE;
+		*searchstr = string + 7;
+	} else if (!strncmp(string, "dependency:", 11)) {
+		*type = NIST_SERVER_DB_SEARCH_DEPENDENCY;
+		*searchstr = string + 11;
+	} else if (!strncmp(string, "address:", 8)) {
+		*type = NIST_SERVER_DB_SEARCH_ADDRESSES;
+		*searchstr = string + 8;
+	} else {
+		logger(LOGGER_ERR, LOGGER_C_ANY,
+		"Unknown search type %s\n", string);
+		return -EINVAL;
+	}
+
+	return 0;
+}
+
 static int parse_fuzzy_flag(bool *fuzzy_search_flag, char **dst,
 			    const char *src)
 {
@@ -643,14 +718,18 @@ static int parse_opts(int argc, char *argv[], struct opt_data *opts)
 			{"list-verdicts",	no_argument,		0, 0},
 			{"list-certificates",	no_argument,		0, 0},
 			{"list-cert-details",	no_argument,		0, 0},
+			{"list-cert-niap",	required_argument,	0, 0},
 			{"list-cipher-options",	no_argument,		0, 0},
 			{"list-cipher-options-deps",no_argument,	0, 0},
+			{"list-server-db",	required_argument,	0, 0},
+			{"search-server-db",	required_argument,	0, 0},
 
 			{"cipher-options",	required_argument,	0, 0},
 			{"cipher-algo",		required_argument,	0, 0},
 			{"cipher-list",		no_argument,		0, 0},
 
 			{"proxy-extension",	required_argument,	0, 0},
+			{"proxy-extension-dir",	required_argument,	0, 0},
 			{"rename-version",	required_argument,	0, 0},
 			{"rename-name",		required_argument,	0, 0},
 			{"rename-oename",	required_argument,	0, 0},
@@ -887,7 +966,7 @@ static int parse_opts(int argc, char *argv[], struct opt_data *opts)
 			case 31:
 				/* delete-definition */
 				CKINT(convert_update_delete_type(optarg,
-				  &opts->acvp_ctx_options.delete_db_entry));
+				      &opts->acvp_ctx_options.delete_db_entry));
 				/* This operation must use the publish path */
 				opts->publish = true;
 				break;
@@ -934,22 +1013,39 @@ static int parse_opts(int argc, char *argv[], struct opt_data *opts)
 				opts->acvp_ctx_options.threading_disabled = true;
 				break;
 			case 40:
+				/* list-cert-niap */
+				opts->list_certificates_detailed = true;
+				opts->acvp_ctx_options.threading_disabled = true;
+				opts->cert_details_niap_req_file = optarg;
+				break;
+			case 41:
 				/* list-cipher-options */
 				opts->list_cipher_options = true;
 				opts->acvp_ctx_options.threading_disabled = true;
 				break;
-			case 41:
+			case 42:
 				/* list-cipher-options-deps */
 				opts->list_cipher_options_deps = true;
 				opts->acvp_ctx_options.threading_disabled = true;
 				break;
+			case 43:
+				/* list-server-db */
+				CKINT(convert_show_type(optarg,
+				      &opts->acvp_ctx_options.show_db_entries));
+				break;
+			case 44:
+				/* search-server-db */
+				CKINT(convert_search_type(optarg,
+						&opts->acvp_server_db_search,
+						&opts->search_type));
+				break;
 
-			case 42:
+			case 45:
 				/* cipher-options */
 				CKINT(duplicate_string(&opts->cipher_options_file,
 						       optarg));
 				break;
-			case 43:
+			case 46:
 				/* cipher-algo */
 				CKINT(duplicate_string(&opts->cipher_options_algo[opts->cipher_options_algo_idx],
 						       optarg));
@@ -960,53 +1056,57 @@ static int parse_opts(int argc, char *argv[], struct opt_data *opts)
 					goto out;
 				}
 				break;
-			case 44:
+			case 47:
 				/* cipher-list */
 				opts->cipher_list = true;
 				break;
 
-			case 45:
+			case 48:
 				/* proxy-extension */
 				CKINT(acvp_load_extension(optarg));
 				break;
-			case 46:
+			case 49:
+				/* proxy-extension-dir */
+				CKINT(acvp_load_extension_directory(optarg));
+				break;
+			case 50:
 				/* rename-version */
 				rename->moduleversion_new = optarg;
 				opts->acvp_ctx_options.threading_disabled = true;
 				opts->rename = true;
 				break;
-			case 47:
+			case 51:
 				/* rename-name */
 				rename->modulename_new = optarg;
 				opts->acvp_ctx_options.threading_disabled = true;
 				opts->rename = true;
 				break;
-			case 48:
+			case 52:
 				/* rename-oename */
 				rename->oe_env_name_new = optarg;
 				opts->acvp_ctx_options.threading_disabled = true;
 				opts->rename = true;
 				break;
-			case 49:
+			case 53:
 				/* rename-procname */
 				rename->proc_name_new = optarg;
 				opts->acvp_ctx_options.threading_disabled = true;
 				opts->rename = true;
 				break;
-			case 50:
+			case 54:
 				/* rename-procseries */
 				rename->proc_series_new = optarg;
 				opts->acvp_ctx_options.threading_disabled = true;
 				opts->rename = true;
 				break;
-			case 51:
+			case 55:
 				/* rename-procfamily */
 				rename->proc_family_new = optarg;
 				opts->acvp_ctx_options.threading_disabled = true;
 				opts->rename = true;
 				break;
 
-			case 52:
+			case 56:
 				/* register-only */
 				opts->acvp_ctx_options.register_only = true;
 				break;
@@ -1204,34 +1304,42 @@ static void last_gen_cb(time_t now)
 	return;
 }
 
-static int set_totp_seed(struct opt_data *opts)
+static int set_totp_seed(struct opt_data *opts, bool enable_net)
 {
 	int ret;
+	char *seed_base64 = NULL;
+	uint32_t seed_base64_len = 0;
 	uint8_t *seed = NULL;
 	uint32_t seed_len;
 	uint64_t totp_last_gen;
 
-	ret = base64_decode(opts->seed_base64, opts->seed_base64_len,
-			    &seed, &seed_len);
-	if (ret) {
-		logger(LOGGER_ERR, LOGGER_C_ANY, "Base64 decoding failed\n");
-		ret = -EFAULT;
-		goto out;
+	if (!enable_net) {
+		return acvp_init(NULL, 0, 0, false, NULL);
 	}
-	memset_secure(opts->seed_base64, 0, opts->seed_base64_len);
+
+	CKINT(load_totp_seed(opts, &seed_base64, &seed_base64_len));
+
+	CKINT_LOG(base64_decode(seed_base64, seed_base64_len,
+			        &seed, &seed_len),
+		  "Base64 decoding failed\n");
 
 	ret = json_get_uint64(opts->config, OPT_STR_TOTPLASTGEN,
 			      &totp_last_gen);
 	if (ret)
 		totp_last_gen = 0;
 
-	CKINT(acvp_init(seed, seed_len, (time_t)totp_last_gen, &last_gen_cb));
+	CKINT(acvp_init(seed, seed_len, (time_t)totp_last_gen,
+			opts->official_testing, &last_gen_cb));
 
 	logger(LOGGER_DEBUG, LOGGER_C_ANY,
 	       "TOTP base64 seed converted into binary and applied\n");
 
 out:
 	/* securely dispose of the seed */
+	if (seed_base64) {
+		memset_secure(seed_base64, 0, seed_base64_len);
+		free(seed_base64);
+	}
 	if (seed) {
 		memset_secure(seed, 0, seed_len);
 		free(seed);
@@ -1239,21 +1347,25 @@ out:
 	return ret;
 }
 
-static int initialize_ctx(struct acvp_ctx **ctx, struct opt_data *opts)
+static int initialize_ctx(struct acvp_ctx **ctx, struct opt_data *opts,
+			  bool enable_net)
 {
 	int ret;
 
+	CKINT(set_totp_seed(opts, enable_net));
+
 	CKINT(acvp_ctx_init(ctx, opts->basedir, opts->secure_basedir));
 
 	/* Official testing */
 	if (opts->official_testing) {
 		CKINT(acvp_req_production(*ctx));
-		CKINT(acvp_set_net(NIST_DEFAULT_SERVER,
-				   NIST_DEFAULT_SERVER_PORT,
+		if (enable_net)
+			CKINT(acvp_set_net(NIST_DEFAULT_SERVER,
+					   NIST_DEFAULT_SERVER_PORT,
 				   opts->tlscabundle, opts->tlscakeychainref,
 				   opts->tlscert, opts->tlscertkeychainref,
 				   opts->tlskey, opts->tlspasscode));
-	} else {
+	} else if (enable_net) {
 		CKINT(acvp_set_net(NIST_TEST_SERVER,
 				   NIST_DEFAULT_SERVER_PORT,
 				   opts->tlscabundle, opts->tlscakeychainref,
@@ -1277,7 +1389,7 @@ static int do_register(struct opt_data *opts)
 	int ret, idx = 0;
 	bool printed = false;
 
-	CKINT(initialize_ctx(&ctx, opts));
+	CKINT(initialize_ctx(&ctx, opts, true));
 
 	ctx->req_details.dump_register = opts->dump_register;
 	ctx->req_details.request_sample = opts->request_sample;
@@ -1326,7 +1438,9 @@ static int do_submit(struct opt_data *opts)
 	int ret, ret2, idx = 0;
 	bool printed = false;
 
-	CKINT(initialize_ctx(&ctx, opts));
+	CKINT(initialize_ctx(&ctx, opts, true));
+
+	ctx->req_details.request_sample = opts->request_sample;
 
 	/*
 	 * We want to list the verdicts we obtained irrespective of
@@ -1375,7 +1489,7 @@ static int do_publish(struct opt_data *opts)
 	struct acvp_ctx *ctx = NULL;
 	int ret;
 
-	CKINT(initialize_ctx(&ctx, opts));
+	CKINT(initialize_ctx(&ctx, opts, true));
 
 	ctx->req_details.dump_register = opts->dump_register;
 
@@ -1391,7 +1505,7 @@ static int list_ids(struct opt_data *opts)
 	struct acvp_ctx *ctx = NULL;
 	int ret;
 
-	CKINT(initialize_ctx(&ctx, opts));
+	CKINT(initialize_ctx(&ctx, opts, false));
 
 	if (opts->list_available_ids) {
 		CKINT(acvp_list_available_ids(ctx));
@@ -1411,7 +1525,7 @@ static int list_verdicts(struct opt_data *opts)
 	struct acvp_ctx *ctx = NULL;
 	int ret;
 
-	CKINT(initialize_ctx(&ctx, opts));
+	CKINT(initialize_ctx(&ctx, opts, false));
 
 	CKINT(acvp_list_verdicts(ctx));
 
@@ -1425,7 +1539,7 @@ static int list_certificates(struct opt_data *opts)
 	struct acvp_ctx *ctx = NULL;
 	int ret;
 
-	CKINT(initialize_ctx(&ctx, opts));
+	CKINT(initialize_ctx(&ctx, opts, false));
 
 	CKINT(acvp_list_certificates(ctx));
 
@@ -1439,7 +1553,7 @@ static int list_cipher_options(struct opt_data *opts)
 	struct acvp_ctx *ctx = NULL;
 	int ret;
 
-	CKINT(initialize_ctx(&ctx, opts));
+	CKINT(initialize_ctx(&ctx, opts, false));
 
 	CKINT(acvp_list_cipher_options(ctx, opts->list_cipher_options_deps));
 
@@ -1453,9 +1567,10 @@ static int list_certificates_detailed(struct opt_data *opts)
 	struct acvp_ctx *ctx = NULL;
 	int ret;
 
-	CKINT(initialize_ctx(&ctx, opts));
+	CKINT(initialize_ctx(&ctx, opts, false));
 
-	CKINT(acvp_list_certificates_detailed(ctx));
+	CKINT(acvp_list_certificates_detailed(ctx,
+					opts->cert_details_niap_req_file));
 
 out:
 	acvp_ctx_release(ctx);
@@ -1467,7 +1582,7 @@ static int do_fetch_cipher_options(struct opt_data *opts)
 	struct acvp_ctx *ctx = NULL;
 	int ret;
 
-	CKINT(initialize_ctx(&ctx, opts));
+	CKINT(initialize_ctx(&ctx, opts, true));
 
 	CKINT(acvp_cipher_get(ctx, opts->cipher_options_algo,
 			      opts->cipher_options_algo_idx,
@@ -1483,7 +1598,7 @@ static int do_rename(struct opt_data *opts)
 	struct acvp_ctx *ctx = NULL;
 	int ret;
 
-	CKINT(initialize_ctx(&ctx, opts));
+	CKINT(initialize_ctx(&ctx, opts, false));
 
 	ctx->rename = &opts->rename_ctx;
 
@@ -1494,6 +1609,35 @@ out:
 	return ret;
 }
 
+static int do_list_server_db(struct opt_data *opts)
+{
+	struct acvp_ctx *ctx = NULL;
+	int ret;
+
+	CKINT(initialize_ctx(&ctx, opts, true));
+
+	CKINT(acvp_server_db_list(ctx));
+
+out:
+	acvp_ctx_release(ctx);
+	return ret;
+}
+
+static int do_search_server_db(struct opt_data *opts)
+{
+	struct acvp_ctx *ctx = NULL;
+	int ret;
+
+	CKINT(initialize_ctx(&ctx, opts, true));
+
+	CKINT(acvp_server_db_search(ctx, opts->search_type,
+				    opts->acvp_server_db_search));
+
+out:
+	acvp_ctx_release(ctx);
+	return ret;
+}
+
 int main(int argc, char *argv[])
 {
 	struct opt_data opts;
@@ -1508,9 +1652,11 @@ int main(int argc, char *argv[])
 
 	CKINT(parse_opts(argc, argv, &opts));
 
-	CKINT(set_totp_seed(&opts));
-
-	if (opts.cipher_list || opts.cipher_options_algo_idx ||
+	if (opts.acvp_server_db_search && opts.search_type) {
+		CKINT(do_search_server_db(&opts));
+	} else if (opts.acvp_ctx_options.show_db_entries) {
+		CKINT(do_list_server_db(&opts));
+	} else if (opts.cipher_list || opts.cipher_options_algo_idx ||
 	    opts.cipher_options_file) {
 		CKINT(do_fetch_cipher_options(&opts));
 	} else if (opts.rename) {
