@@ -446,6 +446,7 @@ static void acvp_def_del_info(struct definition *def)
 
 	ACVP_PTR_FREE_NULL(info->module_name);
 	ACVP_PTR_FREE_NULL(info->impl_name);
+	ACVP_PTR_FREE_NULL(info->impl_description);
 	ACVP_PTR_FREE_NULL(info->orig_module_name);
 	ACVP_PTR_FREE_NULL(info->module_name_filesafe);
 	ACVP_PTR_FREE_NULL(info->module_name_internal);
@@ -557,7 +558,8 @@ out:
 }
 
 static int acvp_def_add_info(struct definition *def, struct def_info *src,
-			     const char *impl_name)
+			     const char *impl_name,
+			     const char *impl_description)
 {
 	struct def_info *info;
 	int ret = 0;
@@ -571,6 +573,7 @@ static int acvp_def_add_info(struct definition *def, struct def_info *src,
 	CKINT(acvp_def_module_name(&info->module_name, src->module_name,
 				   impl_name));
 	CKINT(acvp_duplicate(&info->impl_name, impl_name));
+	CKINT(acvp_duplicate(&info->impl_description, impl_description));
 	CKINT(acvp_duplicate(&info->orig_module_name, src->module_name));
 	CKINT(acvp_duplicate(&info->module_name_filesafe, info->module_name));
 	CKINT(acvp_duplicate(&info->module_name_internal,
@@ -695,7 +698,7 @@ static int acvp_def_add_deps(struct definition *def,
 
 	deps = def->deps;
 	info = def->info;
-	CKNULL_LOG(def, -EINVAL, "Module information context missing\n");
+	CKNULL_LOG(info, -EINVAL, "Module meta data missing\n");
 
 	/* Try finding the dependency for our definition */
 	ret = json_find_key(json_deps, info->impl_name, &my_dep,
@@ -1019,6 +1022,8 @@ static int acvp_def_update_id(const char *pathname,
 	int ret = 0;
 	bool updated = false;
 
+	CKNULL(pathname, -EINVAL);
+
 	CKINT_LOG(acvp_def_read_json(&config, pathname),
 		  "Cannot parse config file %s\n", pathname);
 
@@ -1041,6 +1046,90 @@ out:
 	return ret;
 }
 
+static int acvp_def_set_str(struct json_object *json,
+			    const char *name, const char *str, bool *set)
+{
+	struct json_object *val;
+	int ret;
+
+	if (!name)
+		return -EINVAL;
+
+	/* We may have a string or NULL */
+	ret = json_find_key(json, name, &val, json_type_string);
+	if (ret)
+		ret = json_find_key(json, name, &val, json_type_null);
+
+	/* Create a new entry */
+	if (ret) {
+		/* Do not create a NULL entry */
+		if (!str)
+			return 0;
+
+		json_object_object_add(json, name,
+				       json_object_new_string(str));
+		*set = true;
+		return 0;
+	}
+
+	/* Update existing entry to NULL */
+	if (!str) {
+		/* We do not need updating as the entry is already NULL */
+		if (json_object_is_type(val, json_type_null))
+			return 0;
+
+		json_object_object_del(json, name);
+		json_object_object_add(json, name, NULL);
+		*set = true;
+		return 0;
+	}
+
+	/* Update existing entry if it does not match */
+	if (!acvp_find_match(str, json_object_get_string(val), false)) {
+		json_object_set_string(val, str);
+		*set = true;
+	}
+
+	return 0;
+}
+
+struct acvp_def_update_string_entry {
+	const char *name;
+	const char *str;
+};
+
+static int acvp_def_update_str(const char *pathname,
+			       const struct acvp_def_update_string_entry *list,
+			       uint32_t list_entries)
+{
+	struct json_object *config = NULL;
+	unsigned int i;
+	int ret = 0;
+	bool updated = false;
+
+	CKNULL(pathname, -EINVAL);
+
+	CKINT_LOG(acvp_def_read_json(&config, pathname),
+		  "Cannot parse config file %s\n", pathname);
+
+	for (i = 0; i < list_entries; i++) {
+		if (!list[i].name)
+			continue;
+
+		logger(LOGGER_VERBOSE, LOGGER_C_ANY,
+		       "Updating entry %s with %s\n",
+		       list[i].name, list[i].str);
+		CKINT(acvp_def_set_str(config, list[i].name, list[i].str,
+				       &updated));
+	}
+
+	if (updated)
+		CKINT(acvp_def_write_json(config, pathname));
+
+out:
+	return ret;
+}
+
 #define ACVP_DEF_PRODUCTION_ID(x)					\
 	(acvp_req_is_production() ? x "Production" : x)
 
@@ -1230,6 +1319,34 @@ out:
 	return ret;
 }
 
+int acvp_def_update_oe_config(const struct def_oe *def_oe)
+{
+	struct acvp_def_update_string_entry str_list[7];
+	int ret;
+
+	str_list[0].name = "oeEnvName";
+	str_list[0].str = def_oe->oe_env_name;
+	str_list[1].name = "cpe";
+	str_list[1].str = def_oe->cpe;
+	str_list[2].name = "swid";
+	str_list[2].str = def_oe->swid;
+	str_list[3].name = "manufacturer";
+	str_list[3].str = def_oe->manufacturer;
+	str_list[4].name = "procFamily";
+	str_list[4].str = def_oe->proc_family;
+	str_list[5].name = "procName";
+	str_list[5].str = def_oe->proc_name;
+	str_list[6].name = "procSeries";
+	str_list[6].str = def_oe->proc_series;
+
+	acvp_def_lock_lock(def_oe->def_lock);
+	ret = acvp_def_update_str(def_oe->def_oe_file, str_list,
+				  ARRAY_SIZE(str_list));
+	acvp_def_lock_unlock(def_oe->def_lock);
+
+	return ret;
+}
+
 static int acvp_def_find_module_id(struct def_info *def_info,
 				   struct json_object *config,
 				   struct json_object **entry)
@@ -1387,6 +1504,24 @@ out:
 	return ret;
 }
 
+int acvp_def_update_module_config(const struct def_info *def_info)
+{
+	struct acvp_def_update_string_entry str_list[2];
+	int ret;
+
+	str_list[0].name = "moduleName";
+	str_list[0].str = def_info->orig_module_name;
+	str_list[1].name = "moduleVersion";
+	str_list[1].str = def_info->module_version;
+
+	acvp_def_lock_lock(def_info->def_lock);
+	ret = acvp_def_update_str(def_info->def_module_file, str_list,
+				  ARRAY_SIZE(str_list));
+	acvp_def_lock_unlock(def_info->def_lock);
+
+	return ret;
+}
+
 static int acvp_check_features(uint64_t feature)
 {
 	unsigned int i;
@@ -1481,8 +1616,19 @@ static int acvp_def_load_config(const char *oe_file, const char *vendor_file,
 			      (const char **)&oe.proc_series));
 	CKINT(json_get_uint(oe_config, "features", (uint32_t *)&oe.features));
 	CKINT(acvp_check_features(oe.features));
-	CKINT(json_get_uint(oe_config, "envType", (uint32_t *)&oe.env_type));
-	CKINT(acvp_module_oe_type(oe.env_type, NULL));
+
+	/* Allow numeric or string-based name */
+	ret = json_get_uint(oe_config, "envType", (uint32_t *)&oe.env_type);
+	if (!ret) {
+		CKINT(acvp_module_oe_type(oe.env_type, NULL));
+	} else {
+		const char *oe_type_str;
+		CKINT(json_get_string(oe_config, "envType", &oe_type_str));
+		CKINT_LOG(acvp_module_type_name_to_enum(oe_type_str,
+							&oe.env_type),
+			  "OE envType is allowed to only hold 'Hardware', 'Software' or 'Firmware', but string %s found in file %s\n",
+			  oe_type_str ? oe_type_str : "(null)", oe_file);
+	}
 
 	/* Shall we use the internal name for the mapping lookup? */
 	local_proc_family = oe.proc_family_internal ?
@@ -1546,8 +1692,9 @@ static int acvp_def_load_config(const char *oe_file, const char *vendor_file,
 			      (const char **)&vendor.contact_name));
 	CKINT(json_get_string(vendor_config, "contactEmail",
 			      (const char **)&vendor.contact_email));
-	CKINT(json_get_string(vendor_config, "contactPhone",
-			      (const char **)&vendor.contact_phone));
+	/* no error handling */
+	json_get_string(vendor_config, "contactPhone",
+			(const char **)&vendor.contact_phone);
 	CKINT(json_get_string(vendor_config, "addressStreet",
 			      (const char **)&vendor.addr_street));
 	CKINT(json_get_string(vendor_config, "addressCity",
@@ -1595,8 +1742,7 @@ static int acvp_def_load_config(const char *oe_file, const char *vendor_file,
 
 			string = json_object_get_string(impl);
 
-			if (!strncmp(map->impl_name, string,
-				     strlen(map->impl_name))) {
+			if (acvp_find_match(map->impl_name, string, false)) {
 				found = 1;
 				break;
 			}
@@ -1610,35 +1756,17 @@ static int acvp_def_load_config(const char *oe_file, const char *vendor_file,
 		logger(LOGGER_DEBUG, LOGGER_C_ANY,
 		       "Algorithm map for name %s, processor %s found\n",
 		       info.module_name, oe.proc_family);
-		ret = acvp_def_init(&def, map);
-		if (ret)
-			goto unlock;
-
-		ret = acvp_def_add_oe(def, &oe);
-		if (ret)
-			goto unlock;
-
-		ret = acvp_def_add_vendor(def, &vendor);
-		if (ret)
-			goto unlock;
-
-		ret = acvp_def_add_info(def, &info, map->impl_name);
-		if (ret)
-			goto unlock;
+		CKINT_ULCK(acvp_def_init(&def, map));
+		CKINT_ULCK(acvp_def_add_oe(def, &oe));
+		CKINT_ULCK(acvp_def_add_vendor(def, &vendor));
+		CKINT_ULCK(acvp_def_add_info(def, &info, map->impl_name,
+					     map->impl_description));
 
 		/* First stage of dependencies */
-		ret = acvp_def_load_deps(impl_config, def);
-		if (ret)
-			goto unlock;
-		ret = acvp_def_load_deps(oe_config, def);
-		if (ret)
-			goto unlock;
-		ret = acvp_def_load_deps(vendor_config, def);
-		if (ret)
-			goto unlock;
-		ret = acvp_def_load_deps(info_config, def);
-		if (ret)
-			goto unlock;
+		CKINT_ULCK(acvp_def_load_deps(impl_config, def));
+		CKINT_ULCK(acvp_def_load_deps(oe_config, def));
+		CKINT_ULCK(acvp_def_load_deps(vendor_config, def));
+		CKINT_ULCK(acvp_def_load_deps(info_config, def));
 
 		def->uninstantiated_def = map;
 
@@ -1667,40 +1795,9 @@ out:
 	return ret;
 }
 
-/* Sanity check for path name. */
 static int acvp_def_usable_dirent(struct dirent *dirent)
 {
-	size_t filenamelen, extensionlen;
-	int ret = 0;
-
-	/* Check that entry is neither ".", "..", or a hidden file */
-	if (!strncmp(dirent->d_name, ".", 1))
-		goto out;
-
-	/* Check that it is a regular file or a symlink */
-	if (dirent->d_type != DT_REG && dirent->d_type != DT_LNK)
-		goto out;
-
-	filenamelen = strlen(dirent->d_name);
-	extensionlen = strlen(ACVP_DEF_CONFIG_FILE_EXTENSION);
-
-	/* Check that file name is long enough */
-	if (filenamelen < extensionlen + 1)
-		goto out;
-
-	/* Check for presence of extension */
-	if (strncmp(dirent->d_name + filenamelen - extensionlen,
-		    ACVP_DEF_CONFIG_FILE_EXTENSION, extensionlen))
-		goto out;
-
-	ret = 1;
-
-out:
-	if (!ret)
-		logger(LOGGER_DEBUG, LOGGER_C_ANY,
-		       "Skipping directory entry %s\n", dirent->d_name);
-
-	return ret;
+	return acvp_usable_dirent(dirent, ACVP_DEF_CONFIG_FILE_EXTENSION);
 }
 
 DSO_PUBLIC
