@@ -65,8 +65,8 @@ static int acvp_curl_progress_callback(void *clientp, curl_off_t dltotal,
 	return atomic_bool_read(&acvp_curl_interrupted);
 }
 
-static size_t acvp_curl_read_cb(char *buffer, size_t size, size_t nitems,
-				void *userdata)
+static size_t
+acvp_curl_read_cb(char *buffer, size_t size, size_t nitems, void *userdata)
 {
 	struct acvp_buf *send_buf = (struct acvp_buf *)userdata;
 	size_t sendsize = (size * nitems);
@@ -90,8 +90,8 @@ static size_t acvp_curl_read_cb(char *buffer, size_t size, size_t nitems,
 	return sendsize;
 }
 
-static size_t acvp_curl_write_cb(void *ptr, size_t size, size_t nmemb,
-				 void *userdata)
+static size_t
+acvp_curl_write_cb(void *ptr, size_t size, size_t nmemb, void *userdata)
 {
 	struct acvp_buf *response_buf = (struct acvp_buf *)userdata;
 	size_t bufsize = (size * nmemb);
@@ -143,7 +143,7 @@ static size_t acvp_curl_write_cb(void *ptr, size_t size, size_t nmemb,
 
 static int
 acvp_curl_add_auth_hdr(const struct acvp_auth_ctx *auth,
-			struct curl_slist **slist)
+		       struct curl_slist **slist)
 {
 	size_t bearer_size;
 	char *bearer;
@@ -173,29 +173,24 @@ acvp_curl_add_auth_hdr(const struct acvp_auth_ctx *auth,
  */
 static void acvp_curl_log_peer_cert(CURL *hnd)
 {
-	union {
-		struct curl_slist *to_info;
-		struct curl_certinfo *to_certinfo;
-	} ptr;
+	struct curl_certinfo *to_certinfo = NULL;
 	int i;
 	CURLcode ret;
 
 	if (logger_get_verbosity(LOGGER_C_CURL) < LOGGER_DEBUG2)
 		return;
 
-	ptr.to_info = NULL;
-
-	ret = curl_easy_getinfo(hnd, CURLINFO_CERTINFO, &ptr.to_info);
-	if (ret || !ptr.to_info)
+	ret = curl_easy_getinfo(hnd, CURLINFO_CERTINFO, &to_certinfo);
+	if (ret || !to_certinfo)
 		return;
 
 	logger(LOGGER_DEBUG, LOGGER_C_CURL,
 		"TLS peer presented the following %d certificates...\n",
-		ptr.to_certinfo->num_of_certs);
+		to_certinfo->num_of_certs);
 
-	for (i = 0; i < ptr.to_certinfo->num_of_certs; i++) {
+	for (i = 0; i < to_certinfo->num_of_certs; i++) {
 		struct curl_slist *slist;
-		for (slist = ptr.to_certinfo->certinfo[i];
+		for (slist = to_certinfo->certinfo[i];
 			slist;
 			slist = slist->next) {
 			logger(LOGGER_DEBUG2, LOGGER_C_CURL, "%s\n",
@@ -219,6 +214,7 @@ static int acvp_curl_http_common(const struct acvp_na_ex *netinfo,
 	char useragent[30];
 	int ret;
 	unsigned int retries = 0;
+	long http_response_code = 0;
 
 	CKNULL_LOG(net, -EINVAL, "Network context missing\n");
 	CKNULL_LOG(url, -EINVAL, "URL missing\n");
@@ -369,22 +365,26 @@ static int acvp_curl_http_common(const struct acvp_na_ex *netinfo,
 	CURL_CKINT(curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION,
 				    acvp_curl_write_cb));
 
+#if 0
+	/* Set a specific cipher */
+	CURL_CKINT(curl_easy_setopt(curl, CURLOPT_SSL_CIPHER_LIST,
+				    "ECDHE-RSA-AES128-SHA256"));
+#endif
+
 	/* Perform the HTTP request */
 	while (retries < ACVP_CURL_MAX_RETRIES) {
 		cret = curl_easy_perform(curl);
-		if (cret == CURLE_OK) {
-			ret = 0;
+		if (cret == CURLE_OK)
 			break;
-		}
-
-		ret = -ECONNREFUSED;
 
 		logger(LOGGER_WARN, LOGGER_C_CURL,
 		       "Curl HTTP operation failed with code %d (%s)\n",
 		       cret,  curl_easy_strerror(cret));
 
-		if (cret == CURLE_RECV_ERROR)
-			break;
+		if (cret == CURLE_RECV_ERROR) {
+			ret = -ECONNREFUSED;
+			goto out;
+		}
 
 		retries++;
 		if (retries < ACVP_CURL_MAX_RETRIES) {
@@ -403,20 +403,17 @@ static int acvp_curl_http_common(const struct acvp_na_ex *netinfo,
 		}
 	}
 
-	if (ret)
-		goto out;
-
 	acvp_curl_log_peer_cert(curl);
 
 	/* Get the HTTP response status code from the server */
-	curl_easy_getinfo(curl, CURLINFO_RESPONSE_CODE, &ret);
-	if (ret == HTTP_OK) {
+	curl_easy_getinfo(curl, CURLINFO_RESPONSE_CODE, &http_response_code);
+	if (http_response_code == HTTP_OK) {
 		ret = 0;
 	} else {
 		logger(LOGGER_WARN, LOGGER_C_CURL,
-		       "Unable to HTTP %s data for URL %s: %d\n", http_type_str,
-		       url, ret);
-		ret = -ECONNREFUSED;
+		       "Unable to HTTP %s data for URL %s: %ld\n",
+		       http_type_str, url, http_response_code);
+		ret = -(int)http_response_code;
 	}
 
 out:
