@@ -42,7 +42,7 @@ static int acvp_req_dsa_l_n(const struct def_algo_dsa *dsa,
 		if (dsa->dsa_mode != DEF_ALG_DSA_MODE_SIGVER &&
 		    dsa->dsa_mode != DEF_ALG_DSA_MODE_PQGVER) {
 			logger(LOGGER_WARN, LOGGER_C_ANY,
-			       "L = 1024 only allowed for SigVer\n");
+			       "DSA: L = 1024 only allowed for SigVer\n");
 			ret = -EINVAL;
 			goto out;
 		}
@@ -58,7 +58,8 @@ static int acvp_req_dsa_l_n(const struct def_algo_dsa *dsa,
 					     json_object_new_int(3072)));
 		break;
 	default:
-		logger(LOGGER_WARN, LOGGER_C_ANY, "Unknown DSA L definition\n");
+		logger(LOGGER_WARN, LOGGER_C_ANY,
+		       "DSA: Unknown DSA L definition\n");
 		ret = -EINVAL;
 		goto out;
 	}
@@ -67,7 +68,7 @@ static int acvp_req_dsa_l_n(const struct def_algo_dsa *dsa,
 	case DEF_ALG_DSA_N_160:
 		if (dsa->dsa_l != DEF_ALG_DSA_L_1024) {
 			logger(LOGGER_WARN, LOGGER_C_ANY,
-			       "N = 160 only allowed for L = 1024");
+			       "DSA: N = 160 only allowed for L = 1024");
 			ret = -EINVAL;
 			goto out;
 		}
@@ -77,7 +78,7 @@ static int acvp_req_dsa_l_n(const struct def_algo_dsa *dsa,
 	case DEF_ALG_DSA_N_224:
 		if (dsa->dsa_l != DEF_ALG_DSA_L_2048) {
 			logger(LOGGER_WARN, LOGGER_C_ANY,
-			       "N = 224 only allowed for L = 2048");
+			       "DSA: N = 224 only allowed for L = 2048");
 			ret = -EINVAL;
 			goto out;
 		}
@@ -89,7 +90,8 @@ static int acvp_req_dsa_l_n(const struct def_algo_dsa *dsa,
 					     json_object_new_int(256)));
 		break;
 	default:
-		logger(LOGGER_WARN, LOGGER_C_ANY, "Unknown DSA N definition\n");
+		logger(LOGGER_WARN, LOGGER_C_ANY,
+		       "DSA: Unknown DSA N definition\n");
 		ret = -EINVAL;
 		goto out;
 	}
@@ -98,6 +100,63 @@ out:
 	return ret;
 }
 
+static int acvp_list_dsa_n(const struct def_algo_dsa *dsa,
+			   cipher_t keylen[DEF_ALG_MAX_INT], unsigned int *idx)
+{
+	if (dsa->dsa_n == DEF_ALG_DSA_N_160) {
+		keylen[*idx] = 160;
+		*idx += 1;
+		if (*idx >= DEF_ALG_MAX_INT)
+			return 0;
+	}
+	if (dsa->dsa_n == DEF_ALG_DSA_N_224) {
+		keylen[*idx] = 224;
+		*idx += 1;
+		if (*idx >= DEF_ALG_MAX_INT)
+			return 0;
+	}
+	if (dsa->dsa_n == DEF_ALG_DSA_N_256) {
+		keylen[*idx] = 256;
+		*idx += 1;
+		if (*idx >= DEF_ALG_MAX_INT)
+			return 0;
+	}
+
+	return 0;
+}
+
+static int acvp_list_dsa_l_n(const struct def_algo_dsa *dsa,
+			     cipher_t keylen[DEF_ALG_MAX_INT])
+{
+	unsigned int idx = 0;
+	int ret = 0;
+
+	if (dsa->dsa_l == DEF_ALG_DSA_L_1024) {
+		keylen[idx++] = 1024;
+		CKINT(acvp_list_dsa_n(dsa, keylen, &idx));
+		if (idx >= DEF_ALG_MAX_INT)
+			goto out;
+	}
+	if (dsa->dsa_l == DEF_ALG_DSA_L_2048) {
+		keylen[idx++] = 2048;
+		CKINT(acvp_list_dsa_n(dsa, keylen, &idx));
+		if (idx >= DEF_ALG_MAX_INT)
+			goto out;
+	}
+	if (dsa->dsa_l == DEF_ALG_DSA_L_3072) {
+		keylen[idx++] = 3072;
+		CKINT(acvp_list_dsa_n(dsa, keylen, &idx));
+		if (idx >= DEF_ALG_MAX_INT)
+			goto out;
+	}
+
+	if (idx < DEF_ALG_MAX_INT)
+		keylen[idx] = DEF_ALG_ZERO_VALUE;
+
+out:
+	return ret;
+}
+
 static int acvp_req_dsa_pqggen(const struct def_algo_dsa *dsa,
 			       struct json_object *entry)
 {
@@ -142,7 +201,7 @@ static int acvp_req_dsa_pqggen(const struct def_algo_dsa *dsa,
 	}
 	if (!found) {
 		logger(LOGGER_WARN, LOGGER_C_ANY,
-		       "No gGen information provided\n");
+		       "DSA: No gGen information provided\n");
 		ret = -EINVAL;
 		goto out;
 	}
@@ -151,7 +210,7 @@ static int acvp_req_dsa_pqggen(const struct def_algo_dsa *dsa,
 	    (dsa->dsa_mode != DEF_ALG_DSA_MODE_PQGVER) &&
 	    (dsa->dsa_mode != DEF_ALG_DSA_MODE_SIGVER)) {
 		logger(LOGGER_WARN, LOGGER_C_ANY,
-		       "SHA-1 can only be used with PQGVer or SigVer\n");
+		       "DSA: SHA-1 can only be used with PQGVer or SigVer\n");
 		ret = -EINVAL;
 		goto out;
 	}
@@ -197,6 +256,7 @@ static int acvp_req_dsa_sigver(const struct def_algo_dsa *dsa,
  * Generate algorithm entry for symmetric ciphers
  */
 static int _acvp_req_set_algo_dsa(const struct def_algo_dsa *dsa,
+				  const struct acvp_test_deps *deps,
 				  struct json_object *entry, bool full,
 				  bool publish)
 {
@@ -253,13 +313,13 @@ static int _acvp_req_set_algo_dsa(const struct def_algo_dsa *dsa,
 		break;
 	default:
 		logger(LOGGER_WARN, LOGGER_C_ANY,
-		       "Unknown DSA keygen definition\n");
+		       "DSA: Unknown DSA keygen definition\n");
 		ret = -EINVAL;
 		goto out;
 		break;
 	}
 
-	CKINT(acvp_req_gen_prereq(dsa->prereqvals, dsa->prereqvals_num,
+	CKINT(acvp_req_gen_prereq(dsa->prereqvals, dsa->prereqvals_num, deps,
 				  entry, publish));
 
 out:
@@ -268,14 +328,61 @@ out:
 	return ret;
 }
 
+int acvp_list_algo_dsa(const struct def_algo_dsa *dsa,
+		       struct acvp_list_ciphers **new)
+{
+	struct acvp_list_ciphers *tmp = NULL;
+	int ret = 0;
+
+	tmp = calloc(1, sizeof(struct acvp_list_ciphers));
+	CKNULL(tmp, -ENOMEM);
+	*new = tmp;
+
+	CKINT(acvp_duplicate(&tmp->cipher_name, "DSA"));
+	CKINT(acvp_list_dsa_l_n(dsa, tmp->keylen));
+	tmp->prereqs = dsa->prereqvals;
+	tmp->prereq_num = dsa->prereqvals_num;
+
+	switch (dsa->dsa_mode) {
+	case DEF_ALG_DSA_MODE_PQGGEN:
+		CKINT(acvp_duplicate(&tmp->cipher_mode, "pqgGen"));
+		break;
+	case DEF_ALG_DSA_MODE_PQGVER:
+		CKINT(acvp_duplicate(&tmp->cipher_mode, "pqgVer"));
+		break;
+	case DEF_ALG_DSA_MODE_KEYGEN:
+		CKINT(acvp_duplicate(&tmp->cipher_mode, "keyGen"));
+		break;
+	case DEF_ALG_DSA_MODE_SIGGEN:
+		CKINT(acvp_duplicate(&tmp->cipher_mode, "sigGen"));
+		break;
+	case DEF_ALG_DSA_MODE_SIGVER:
+		CKINT(acvp_duplicate(&tmp->cipher_mode, "sigVer"));
+		break;
+	default:
+		logger(LOGGER_WARN, LOGGER_C_ANY,
+		       "DSA: Unknown DSA keygen definition\n");
+		ret = -EINVAL;
+		goto out;
+		break;
+	}
+
+	CKINT(acvp_req_cipher_to_stringarray(dsa->hashalg, ACVP_CIPHERTYPE_HASH,
+					     &tmp->cipher_aux));
+
+out:
+	return ret;
+}
+
 int acvp_req_set_prereq_dsa(const struct def_algo_dsa *dsa,
+			    const struct acvp_test_deps *deps,
 			    struct json_object *entry, bool publish)
 {
-	return _acvp_req_set_algo_dsa(dsa, entry, false, publish);
+	return _acvp_req_set_algo_dsa(dsa, deps, entry, false, publish);
 }
 
 int acvp_req_set_algo_dsa(const struct def_algo_dsa *dsa,
 			  struct json_object *entry)
 {
-	return _acvp_req_set_algo_dsa(dsa, entry, true, false);
+	return _acvp_req_set_algo_dsa(dsa, NULL, entry, true, false);
 }
