@@ -68,6 +68,15 @@ static DEFINE_MUTEX_W_UNLOCKED(totp_lock);
  */
 static atomic_bool_t totp_shutdown = ATOMIC_BOOL_INIT(false);
 
+#if (GCC_VERSION >= 40400) || defined(__clang__)
+# define __HAVE_BUILTIN_BSWAP32__
+# define __HAVE_BUILTIN_BSWAP64__
+#endif
+
+#ifdef __HAVE_BUILTIN_BSWAP64__
+# define _swap64(x) (uint64_t)__builtin_bswap64((uint64_t)(x))
+#else
+
 /****************
  * Rotate the 32 bit unsigned integer X by N bits left/right
  */
@@ -93,14 +102,6 @@ static inline uint64_t _bswap64(uint64_t x)
 		(uint64_t)(_bswap32((uint32_t)(x >> 32)));
 }
 
-#if GCC_VERSION >= 40400
-# define __HAVE_BUILTIN_BSWAP32__
-# define __HAVE_BUILTIN_BSWAP64__
-#endif
-
-#ifdef __HAVE_BUILTIN_BSWAP64__
-# define _swap64(x) (uint64_t)__builtin_bswap64((uint64_t)(x))
-#else
 # define _swap64(x) _bswap64(x)
 #endif
 
@@ -319,7 +320,7 @@ static int totp_protection(void)
 }
 
 int totp_set_seed(const uint8_t *K, uint32_t Klen, time_t last_gen,
-		  void (*last_gen_cb)(time_t now))
+		  bool production, void (*last_gen_cb)(time_t now))
 {
 	int ret;
 
@@ -351,7 +352,7 @@ int totp_set_seed(const uint8_t *K, uint32_t Klen, time_t last_gen,
 	totp_last_generated = last_gen;
 	totp_last_gen_cb = last_gen_cb;
 
-	ret = totp_mq_init();
+	ret = totp_mq_init(production);
 
 out:
 	mutex_w_unlock(&totp_lock);
